package com.sd.business.service.inventory.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByDepartmentDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.dto.QuantityDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.mapper.inventory.InventoryMapper;
import com.sd.business.service.inventory.InventoryService;
import com.sd.framework.util.excel.util.ExcelUtil;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {

    @Autowired
    private HttpServletResponse response;

    @Override
    public Page<InventoryVo> getPage(InventorySelectDto dto) {

        Page<InventoryVo> page = Sql.create(InventoryVo.class)
                .selectAll(Inventory.class)
                .selectAs(Warehouse::getName, InventoryVo::getWarehouseName)
                .selectAs(BomSpec::getCode, InventoryVo::getBomSpecCode)
                .selectAs(BomSpec::getName, InventoryVo::getBomSpecName)
                .selectAs(Department::getName, InventoryVo::getDepartmentName)
                .multiply(Inventory::getQuantity, Inventory::getBalanceUnitPrice, InventoryVo::getTotalPrice)
                .add(Inventory::getQuantity, Inventory::getLockQuantity, InventoryVo::getTotalQuantity)

                .from(Inventory.class)
                .leftJoin(BomSpec.class, BomSpec::getId, Inventory::getBomSpecId)
                .leftJoin(Bom.class, Bom::getId, BomSpec::getBomId)
                .leftJoin(Warehouse.class, Warehouse::getId, Inventory::getWarehouseId)
                .leftJoin(Department.class, Department::getId, Inventory::getDepartmentId)

                .orderByDesc(Inventory::getId)
                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .in(Inventory::getWarehouseId, dto.getWarehouseIdList())
                .like(BomSpec::getName, dto.getBomSpecName())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getColour, dto.getBomSpecColour())
                .like(BomSpec::getLength, dto.getBomSpecLength())
                .like(BomSpec::getWidth, dto.getBomSpecWidth())
                .eq(Bom::getBomClassifyId, dto.getBomClassifyId())
                .eq(Bom::getSpecies, dto.getBomSpecies())
                .eq(Bom::getChromatophore, dto.getBomChromatophore())
                .eq(Bom::getEmbossingProcess, dto.getBomEmbossingProcess())
                .eq(Bom::getFrontGrain, dto.getBomFrontGrain())
                .eq(Bom::getReverseGrain, dto.getBomReverseGrain())
                .page(dto);

        return page;
    }

    @Override
    public void exportExcel(InventorySelectDto dto) {
        dto.setSearchAll(true);
        Page<InventoryVo> page = getPage(dto);
        List<InventoryVo> list = page.getRecords();
        ExcelUtil.export(response, DateUtil.format(new Date(), "yyyy-MM-dd库存数据"), "库存数据", list, InventoryVo.class);
    }

    @Override
    public Map<Long, BigDecimal> getQuantity(QuantityDto dto) {
        return mapKV(Inventory::getBomSpecId, Inventory::getQuantity, q -> q
                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .in(Inventory::getBomSpecId, dto.getBomSpecIdList()));
    }

    @Override
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto) {

        return Sql.create(QuantityByWarehouseVo.class)
                .sum(Inventory::getQuantity, QuantityByWarehouseVo::getInventoryQuantity)
                .selectAs(Warehouse::getId, QuantityByWarehouseVo::getWarehouseId)
                .selectAs(Warehouse::getName, QuantityByWarehouseVo::getWarehouseName)

                .from(Inventory.class)
                .innerJoin(Warehouse.class, Warehouse::getId, Inventory::getId)

                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .groupBy(Warehouse::getId)
                .orderByAsc(Warehouse::getId)
                .list();
    }

    @Override
    public List<QuantityByDepartmentVo> getQuantityByDepartment(QuantityByDepartmentDto dto) {

        return Sql.create(QuantityByDepartmentVo.class)
                .sum(Inventory::getQuantity, QuantityByDepartmentVo::getInventoryQuantity)
                .selectAs(Department::getId, QuantityByDepartmentVo::getDepartmentId)
                .selectAs(Department::getName, QuantityByDepartmentVo::getDepartmentName)

                .from(Inventory.class)
                .innerJoin(Department.class, Department::getId, Inventory::getDepartmentId)

                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .groupBy(Department::getId)
                .orderByAsc(Department::getId)
                .list();
    }

}
